# Copyright (C) 2014-2021, Josef Hahn
#
# This file is part of anise.
#
# anise is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# anise is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with anise.  If not, see <http://www.gnu.org/licenses/>.

import os

import anise.framework.features
import anise.framework.projects

from anise.framework.projects import universe

projectdescriptioneditor = anise.framework.features.loadfeature("projectdescriptioneditor").featuremodule


# noinspection PyUnusedLocal
@anise.framework.features.hookhandler(projectdescriptioneditor.HOOK_GET_FEATURE_ACTIONS)
class ProjectDescriptionEditorAssistant(projectdescriptioneditor.ProjectDescriptionEditorAssistant):

    def __init__(self):
        projectdescriptioneditor.ProjectDescriptionEditorAssistant.__init__(
            self, "Manage dependencies", "Specify which other products are required for using this project. This "
                                         "information is used for content generators, like for documentation or "
                                         "installation packages.", "dependencies")

    def getcustomactions(self, ais):
        return [projectdescriptioneditor.CustomAction("Add new dependency", self._add_dependency,
                                                      icon="userfeedback/add.png")]

    def _add_dependency(self, ais):
        objectname = universe.ui.userfeedback.input("Please enter a name for the dependency.", "Flux-Compensator")
        if objectname is None:
            return
        icons = sorted([x[:-4] for x in os.listdir(universe.specialpaths.anisedatadir+"/icons")
                        if x.endswith(".png")])
        icon = universe.ui.userfeedback.choice("Please choose an icon.", icons)
        if icon is None:
            return
        comment = universe.ui.userfeedback.input("Please enter a comment.")
        if comment is None:
            return
        dtypes = [x for x in dir(universe.dependencies.Type) if not x.startswith("_")]
        dtype = universe.ui.userfeedback.choice("Please choose a dependency type.", dtypes)
        if dtype is None:
            return
        oprms = {}
        if len(comment.strip()) > 0:
            oprms["comment"] = anise.framework.projects.IntermediateStructure.Value.fromstring(comment)
        vtype = anise.framework.projects.IntermediateStructure.Value(
            "objectref", oref="dependencies.Type."+dtypes[dtype])
        vdep = anise.framework.projects.IntermediateStructure.Value(
            "dependencies.Dependency", type=vtype,
            objectname=anise.framework.projects.IntermediateStructure.Value.fromstring(objectname),
            icon=anise.framework.projects.IntermediateStructure.Value.fromstring(icons[icon]), **oprms)
        newlet = anise.framework.projects.IntermediateStructure.Let(k="dependencies.pool.add", dependency=vdep)
        ais.addentry(newlet)

    def getentries(self, ais):
        r = []
        for e in ais.entries():
            if isinstance(e, anise.framework.projects.IntermediateStructure.Let) and e.key == "dependencies.pool.add":
                r.append(e)
        return r

    def getentrylabel(self, ais, entry):
        edeps = entry.getarg("dependency")
        if "objectname" in edeps.args():
            return edeps.getarg("objectname").value
        else:
            return edeps.vtype
