# Copyright (C) 2014-2021, Josef Hahn
#
# This file is part of anise.
#
# anise is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# anise is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with anise.  If not, see <http://www.gnu.org/licenses/>.

import anise.framework.features
import anise.framework.projects

from anise.framework.projects import universe

HOOK_GET_FILETRANSFER_PROVIDERS = anise.framework.features.Hook(
    hide=True, doc="This hook is called for collecting configuration guide helpers for file transfers.")


def ask_new_filetransfer(msg):
    impls = [x() for x in universe.gethooks(HOOK_GET_FILETRANSFER_PROVIDERS)]
    implnames = [x.label for x in impls]
    i = universe.ui.userfeedback.choice(msg, implnames)
    if i is not None:
        return impls[i].ask()


@anise.framework.features.hookhandler(HOOK_GET_FILETRANSFER_PROVIDERS)
class LocalFiletransfer:

    def __init__(self):
        self.label = "transfer to local filesystem"

    def ask(self):
        destination = universe.ui.userfeedback.input("Please enter a local filesystem path as transfer destination.")
        if destination is None:
            return None
        return anise.framework.projects.IntermediateStructure.Value(
            "dict", destination=anise.framework.projects.IntermediateStructure.Value.fromstring(destination),
            task=anise.framework.projects.IntermediateStructure.Value("objectref", oref="filetransfer.localupload"))


@anise.framework.features.hookhandler(HOOK_GET_FILETRANSFER_PROVIDERS)
class SshFiletransfer:

    def __init__(self):
        self.label = "transfer to ssh filesystem"

    def ask(self):
        host = universe.ui.userfeedback.input("Please enter the destination hostname.")
        if host is None:
            return None
        username = universe.ui.userfeedback.input("Please enter the ssh user account name.")
        if username is None:
            return None
        destination = universe.ui.userfeedback.input("Please enter the destination path on the destination host.")
        if destination is None:
            return None
        port = None
        while port is None:
            port = universe.ui.userfeedback.input("Please enter the ssh port number.", "22")
            if port is None:
                return None
            try:
                port = int(port)
            except ValueError:
                port = None
        identityfile = universe.ui.userfeedback.input("Please enter the path to an ssh identity file (optional).")
        if identityfile is None:
            return None
        prms = {}
        if len(identityfile.strip()) > 0:
            prms["identityfile"] = anise.framework.projects.IntermediateStructure.Value.fromstring(identityfile)
        if port != 22:
            prms["port"] = anise.framework.projects.IntermediateStructure.Value("int", v=port)
        return anise.framework.projects.IntermediateStructure.Value(
            "dict", host=anise.framework.projects.IntermediateStructure.Value.fromstring(host),
            username=anise.framework.projects.IntermediateStructure.Value.fromstring(username),
            destination=anise.framework.projects.IntermediateStructure.Value.fromstring(destination),
            task=anise.framework.projects.IntermediateStructure.Value("objectref", oref="filetransfer.sshupload"),
            **prms)
