# Copyright (C) 2014-2021, Josef Hahn
#
# This file is part of anise.
#
# anise is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# anise is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with anise.  If not, see <http://www.gnu.org/licenses/>.

"""
Feature for unit testing with pyunit.
"""

from typing import *

import anise.framework.features
import anise.framework.files
import anise.utils

from anise.framework.projects import universe

testinggeneric = anise.framework.features.loadfeature("testing.generic").featuremodule


def pytesttests(testrun: 'testing.Internals.TestRun', testdir: Optional[str] = None) -> None:
    """
    Runs all pytest tests in the project. The discovery is done automatically.

    :param testrun: A test run.
    :param testdir: The root directory where to look for tests.
    """
    with anise.framework.files.TempDir() as tempdir:
        with anise.utils.basic.ChDir(tempdir.path):
            testinggeneric.generictest(testrun, ["python3", "-m", "pytest",
                                                 f"{universe.specialpaths.projectdir}/{testdir}"])


if TYPE_CHECKING:
    import anise.features.testing as testing
