/*
 @licstart  The following is the entire license notice for the JavaScript code in this page.

 Copyright (C) 2014-2021, Josef Hahn

 This file is part of anise.

 anise is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 anise is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with anise.  If not, see <http://www.gnu.org/licenses/>.

 @licend  The above is the entire license notice for the JavaScript code in this page.
*/

anise = new Object();
anise_internalUI = new Object();
(function(){

    class ProgressIndicator {

        constructor(domnode, skipText, progressClass) {
            this.domnode = domnode;
            this.skipText = skipText;
            this.progressClass = domnode.className + " " + (progressClass ? progressClass : "anise_progressviewer") + " ";
            this.setProgressIndeterminate();
        }

        setProgressFinished() {
            if (!this.skipText)
                this.domnode.innerHTML = "&#10003;";
            this.domnode.className = "anise_progressviewer_finished " + this.progressClass;
        }

        setProgressDeterminate(val) {
            this.domnode.className = "anise_progressviewer_determinate " + this.progressClass;
            if (!this.skipText) {
                this.domnode.innerHTML = "";
                for(var i=1;i<=10;i++)
                {
                    var segm = document.createElement("span");
                    segm.className = "anise_progressviewer_inner " + ((val*10 >= i) ? "anise_progressviewer_done" : "anise_progressviewer_undone");
                    this.domnode.appendChild(segm);
                }
            }
        }

        setProgressIndeterminate() {
            if (!this.skipText)
                this.domnode.innerHTML = "&#10022;";
            this.domnode.className = "anise_progressviewer_indeterminate " + this.progressClass;
        }

        setProgressFailed() {
            if (!this.skipText)
                this.domnode.innerHTML = "&#215;";
            this.domnode.className = "anise_progressviewer_failed " + this.progressClass;
        }

        setProgressDoneWithUnknownSuccess() {
            if (!this.skipText)
                this.domnode.innerHTML = "";
            this.domnode.className = this.progressClass;
        }

        set(progressdata) {
            if (progressdata.kind == "indeterminate")
                this.setProgressIndeterminate();
            else if (progressdata.kind == "determinate")
                this.setProgressDeterminate(progressdata.value);
            else if (progressdata.kind == "failed")
                this.setProgressFailed();
            else if (progressdata.kind == "finished")
                this.setProgressFinished();
            else if (progressdata.kind == "donewithunknownsuccess")
                this.setProgressDoneWithUnknownSuccess();
            else
                throw new Error("Invalid progress kind: " + progressdata.kind);
        }

    }

    anise.ProgressIndicator = ProgressIndicator;

    class ExecPagerManager {

        constructor(pager) {
            var self = this;
            this._lastid = -1;
            this.pager = pager;
            this.pullContent();
            yaji.addEventHandler("anise_ExecPager_newcontentavailable", function(e){
                self.pullContent();
            });
        }

        pullContent() {
            var self = this;
            if (this._pulling) {
                this._pullagain = true;
                return;
            }
            this._pulling = true;
            yaji.ajax({
                url: 'getprogresses',
                alsoAfterShutdown: true,
                data: {since: this._lastid},
                onsuccess: function(data) {
                    var newlastid = self.pager.refresh(data);
                    self._lastid = Math.max(newlastid, self._lastid);
                    setTimeout(function() { self.pullContent(); }, self._pullagain?200:2000);
                    self._pulling = false;
                    self._pullagain = false;
                }
            });
        }

    }

    class ExecPager extends clove.Widget {

        constructor(config, domnode) {
            super(clove.utils.applyDefaultsToConfig(config, {
                hstretch: 1,
                vstretch: 1,
                width: '1px',
                height: '1px',
            }), domnode);
            var self = this;
            this._refreshitems_lastrunningscopes = [];
            this.declareProperty("debugVisible", false);
            this.declareProperty("rootScopeAddress");
            this.addStyleClass("anise_execpager");
            $(this.contentnode).addClass("anise_execpager_body");
            this.outputpanels = {};
            this.outputpanels[String([])] = this.contentnode;
            this.manager = new ExecPagerManager(this);
            this.OnSetContent = new clove.Event();
            this.OnSetProgress = new clove.Event();
            this._fixedhead = document.createElement("div");
            this._fixedhead.className = "anise_execpager_fixedhead";
            this.topnode.appendChild(this._fixedhead);
            this._fixedhead_inner = document.createElement("div");
            this._fixedhead_inner.className = "anise_execpager_fixedhead_inner";
            this._fixedhead.appendChild(this._fixedhead_inner);
            this._fixedhead_shadow = document.createElement("div");
            this._fixedhead_shadow.className = "anise_execpager_fixedhead_shadow";
            this._fixedhead.appendChild(this._fixedhead_shadow);
            function setgeom() {
                self._bottompaddingpx = clove.utils.cssLengthToPixels('5pt', 'y');
            }
            $(this.contentnode).on('scroll', function(){
                self._refresh_fixedhead();
            });
            this.OnResized.addHandler(function(){
                setgeom();
                self._refresh_fixedhead();
            });
            setgeom();
            this._refresh_fixedhead();
        }

        debugVisible_setter(v) {
            var isscrolleddown = this.isScrolledDown();
            this.setStyleClassAssigned("anise_execpager_showlogdebug", v);
            this.setStyleClassAssigned("anise_execpager_hidelogdebug", !v);
            if (isscrolleddown)
                $(this.contentnode).scrollTop(this.contentnode.scrollHeight);
        }

        _tuplestartswith(larger, shorter)
        {
            if (larger.length <= shorter.length)
                return false;
            for(var i=0; i<shorter.length; i++)
            {
                if (shorter[i] != larger[i])
                    return false;
            }
            return true;
        }

        _refresh_fixedhead() {
            this._fixedhead.style.left = this.contentnode.clientLeft + "px";
            this._fixedhead.style.width = this.contentnode.clientWidth + "px";
            var rn = this._fixedhead_inner;
            if (this._currentscope) {
                var lastrunningscopes = this._refreshitems_lastrunningscopes;
                this._refreshitems_lastrunningscopes = [];
                for(var i=1; i<=this._currentscope.length; i++) {
                    var iaddr = this._currentscope.slice(0, i);
                    if (!this.config.rootScopeAddress || this._tuplestartswith(iaddr, this.config.rootScopeAddress)) {
                        var inode = this.outputpanels[String(iaddr)];
                        if (inode.offsetTop < this.contentnode.scrollTop + rn.offsetTop /* this is our current eff. height */) {
                            var fchld = rn.firstChild;
                            if (!fchld || fchld.address != String(iaddr)) {
                                rn.innerHTML = "";
                                fchld = document.createElement("div");
                                fchld.className = "anise_execpager_execscope";
                                rn.appendChild(fchld);
                                fchld._label = document.createElement("h1");
                                fchld.appendChild(fchld._label);
                                fchld._inner = document.createElement("div");
                                fchld.appendChild(fchld._inner);
                                fchld.address = String(iaddr);
                                fchld._label.textContent = inode._title;
                            }
                            rn = fchld._inner;
                        }
                        $(inode).addClass("anise_execpager_execscope_running");
                        this._refreshitems_lastrunningscopes.push(inode);
                        var ilst = lastrunningscopes.indexOf(inode);
                        if (ilst > -1)
                            lastrunningscopes.splice(ilst, 1);
                    }
                }
                for(let lnode of lastrunningscopes)
                    $(lnode).removeClass("anise_execpager_execscope_running");
            }
            rn.innerHTML = "";
            var hh = rn.offsetTop;
            this._fixedhead.style.height = (hh + this._bottompaddingpx) + "px";
            this._fixedhead.style.opacity = (!hh) ? 0 : 1;
        }

        _refreshitems(items) {
            var maxid = -1;
            for (var iitem in items)
            {
                var item = items[iitem];
                maxid = Math.max(maxid, item.id);
                if (item.id == 0)
                    continue;
                if (item.title)
                {
                    var newnode = document.createElement("div");
                    newnode.className = "anise_execpager_execscope";
                    var newnodeh1 = document.createElement("h1");
                    newnode.appendChild(newnodeh1);
                    var newnodeprogressindicator = document.createElement("span");
                    newnode.progressindicator = new anise.ProgressIndicator(newnodeprogressindicator);
                    newnodeh1.appendChild(newnodeprogressindicator);
                    var newnodeh1title = document.createElement("span");
                    newnodeh1title.textContent = newnode._title = item.title;
                    newnodeh1.appendChild(newnodeh1title);
                } else {
                    var newnode = document.createElement("span");
                    newnode.textContent = item.value;
                }
                var parentaddress = item.address.slice();
                parentaddress.pop();
                var parentnode = this.outputpanels[String(parentaddress)];
                parentnode.appendChild(newnode);
                this.outputpanels[String(item.address)] = newnode;
                if (this.config.rootScopeAddress && !this._tuplestartswith(item.address, this.config.rootScopeAddress))
                    newnode.className += " anise_execpager_execscope_notvisible";
                for(var i in item.hints)
                    newnode.className += " anise_execpager_item_" + item.hints[i];
                this.OnSetContent.trigger({widget:this, address:String(item.address), item:item});
            }
            return maxid;
        }

        _refreshprogresses(progresses) {
            var doneaddresses = [];
            for (var i in this.outputpanels)
                doneaddresses.push(i);
            for(var iprg in progresses)
            {
                var prg = progresses[iprg];
                var addr = prg[0];
                var saddr = String(addr);
                var val = prg[1];
                doneaddresses.splice(doneaddresses.indexOf(saddr), 1);
                var part = this.outputpanels[saddr];
                if (part.progressindicator)
                    part.progressindicator.set(val);
                this.OnSetProgress.trigger({widget:this, address:saddr, progress:val});
            }
            for(var idoneaddress in doneaddresses)
            {
                var doneaddress = doneaddresses[idoneaddress];
                var part = this.outputpanels[doneaddress];
                if (part.progressindicator)
                    part.progressindicator.setProgressFinished();
                this.OnSetProgress.trigger({widget:this, address:doneaddress, progress:{kind: 'finished'}});
            }
        }

        isScrolledDown() {
            var contentheight = this.contentnode.scrollHeight;
            var containerheight = clove.utils.getOuterSize(this.topnode).height;
            var scrollpos = this.contentnode.scrollTop;
            return (contentheight - containerheight - scrollpos) < 10;
        }

        scrollDown() {
            $(this.contentnode).scrollTop(this.contentnode.scrollHeight);
        }

        refresh(data) {
            this._currentscope = data.currentscope;
            var isscrolleddown = this.isScrolledDown();
            var maxid = this._refreshitems(data.items);
            this._refreshprogresses(data.progresses);
            if (isscrolleddown)
                this.scrollDown();
            this._refresh_fixedhead();
            return maxid;
        }

    }

    anise.ExecPager = ExecPager;

    class FeedbackRequest {

        static _from_simple_repr_(data) {
            return data;
        }

        static answer(index, answer) {
            yaji.ajax({
                url: 'answeruserfeedback',
                data: { index: index, answer: JSON.stringify([answer]) },
                method: 'POST',
            });
        }

    }

    anise.FeedbackRequest = FeedbackRequest;

    class UserFeedbackButton extends clove.Button {

        constructor(config, domnode) {
            super(config, domnode);
            this._a_shortcutsymbol = document.createElement("span");
            this._a_shortcutsymbol.className = "anise_userfeedback_shortcuttext";
            $(this._a_label).after(this._a_shortcutsymbol);
            this.declareProperty("shortcutText");
        }

        shortcutText_setter(v) {
            this._a_shortcutsymbol.style.visibility = v ? "visible" : "collapse";
            this._a_shortcutsymbol.textContent = v;
        }

    }

    anise_internalUI.UserFeedbackButton = UserFeedbackButton;

    class UserFeedbackWidget extends clove.NameScope(clove.StackLayout(clove.Widget)) {

        constructor(config, domnode) {
            super(clove.utils.applyDefaultsToConfig(config, {
                orientation: 'horizontal',
                children: [
                    { view: 'IconView', styleClass: 'anise_userfeedback_margin', name: 'mainicon', strictVerticalSizing: true },
                    { rows: [
                        { rows: [], name: 'body', vstretch: 1, hstretch: 1 },
                        { view: 'Wrap', name: 'buttons' },
                    ], vstretch: 1, hstretch: 1 },
                ],
                hstretch: 1, vstretch: 1, mayFocus: true,
            }), domnode);
            var self = this;
            this.declareProperty("body", []);
            this.declareProperty("mainicon", undefined);
            this.declareProperty("buttons", []);
            this.declareProperty("data", []);
            this._shortcuts = new Object();
            this.allowedshortcuts = "abcdefghijklmnopqrstuvwxyz1234567890";
            this.OnKeyDown.addHandler(function(e){
                var c = self._shortcuts[String(e.keyCode)];
                if (c)
                    c();
            });
        }

        addShortcut(fct, key, code) {
            if (key === undefined) {
                for(var ikey=0; ikey<this.allowedshortcuts.length; ikey++) {
                    var _key = this.allowedshortcuts[ikey];
                    code = String(_key.toUpperCase().charCodeAt(0));
                    if (!this._shortcuts.hasOwnProperty(code)) {
                        key = _key;
                        break;
                    }
                }
            }
            code = String(code);
            if (key !== undefined) {
                this._shortcuts[code] = fct;
                if (code == "10")
                    this._shortcuts["13"] = fct;
            }
            return key;
        }

        answerFeedback(answer) {
            FeedbackRequest.answer(this.getProperty("idx"), answer);
        }

        mainicon_setter(v) {
            this.getByName('mainicon').setIcon(v ? clove.Icon.byUrl("static/icons/"+v) : undefined);
        }

        body_setter(v) {
            this.getByName('body').setChildren(v);
        }

        buttons_setter(v) {
            this.getByName('buttons').setChildren(v);
        }

    }

    class UserFeedbackWidget_message extends UserFeedbackWidget {

        data_setter(data) {
            var self = this;
            var buttons = [];
            for(var i=0; i<data.btns.length; i++) {
                var fct = (function(_i){ return function(){self.answerFeedback(_i);}; })(i);
                buttons.push({ view: 'anise_internalUI.UserFeedbackButton', shortcutText: this.addShortcut(fct), styleClass: 'anise_userfeedback_smallmargin', label: data.btns[i], OnClicked: fct});
            }
            this.setMainicon(data.image);
            this.setBody([{label: data.msg, styleClass: 'anise_userfeedback_scrollable anise_userfeedback_larger', vstretch: 1, hstretch: 1}]);
            this.setButtons(buttons);
        }

        doinit() {
            super.doinit();
            var self = this;
            setTimeout(function(){self.focus();});
        }

    }

    anise_internalUI.UserFeedbackWidget_message = UserFeedbackWidget_message;

    class UserFeedbackWidget_input extends UserFeedbackWidget {

        constructor(config, domnode) {
            super(config, domnode);
            this._inttype = 'EditBox';
            this._intverticalStretchAffinity = 0;
            this._keyboardshortcuts = true;
        }

        data_setter(data) {
            var self = this;
            var stok;
            var stc;
            var okfct = function() {
                self.answerFeedback(self.getByName('edit').text());
            };
            var cancelfct = function() {
                self.answerFeedback(undefined);
            };
            if (this._keyboardshortcuts) {
                stok = this.addShortcut(okfct, 'enter', 10);
                stc = this.addShortcut(cancelfct, 'esc', 27);
            }
            this.setBody([
                { label: data.msg, styleClass: 'anise_userfeedback_scrollable anise_userfeedback_larger' },
                { view: this._inttype, name: 'edit', text: data.default, styleClass: 'anise_userfeedback_margin', hstretch: 1, vstretch: this._intverticalStretchAffinity },
            ]);
            this.setButtons([
                {view: 'anise_internalUI.UserFeedbackButton', shortcutText: stok, styleClass: 'anise_userfeedback_smallmargin', label: 'OK', OnClicked: okfct},
                {view: 'anise_internalUI.UserFeedbackButton', shortcutText: stc, styleClass: 'anise_userfeedback_smallmargin', label: 'Cancel', OnClicked: cancelfct},
            ]);
        }

        doinit() {
            super.doinit();
            var self = this;
            setTimeout(function(){self.getByName('edit').focus();});
        }

    }

    anise_internalUI.UserFeedbackWidget_input = UserFeedbackWidget_input;

    class UserFeedbackWidget_multilineinput extends UserFeedbackWidget_input {

        constructor(config, domnode) {
            super(config, domnode);
            this._inttype = 'MultilineEditBox';
            this._intverticalStretchAffinity = 1;
            this._keyboardshortcuts = false;
        }

    }

    anise_internalUI.UserFeedbackWidget_multilineinput = UserFeedbackWidget_multilineinput;

    class UserFeedbackWidget_choice extends UserFeedbackWidget {

        data_setter(data) {
            var self = this;
            var body = [];
            for(var i=0; i<data.choices.length; i++) {
                var fct = (function(_i){ return function(){self.answerFeedback(_i);}; })(i);
                var bicon = (data.choiceimages && data.choiceimages.length > i && data.choiceimages[i]) ? clove.Icon.byUrl("static/icons/"+data.choiceimages[i]) : undefined;
                body.push({ view: 'anise_internalUI.UserFeedbackButton', shortcutText: this.addShortcut(fct), label: data.choices[i], icon: bicon, styleClass: 'anise_userfeedback_smallmargin', OnClicked: fct });
            }
            this.setBody([
                {label: data.msg, styleClass: 'anise_userfeedback_scrollable anise_userfeedback_larger'},
                {rows: body, styleClass: 'anise_userfeedback_scrollable anise_userfeedback_margin', vstretch: 1, height: "1pt", hstretch: 1, width: "1pt"},
            ]);
            var cancelfct = function(){self.answerFeedback(-1);}
            this.setButtons([{ view: 'anise_internalUI.UserFeedbackButton', shortcutText: this.addShortcut(cancelfct, 'esc', 27), styleClass: 'anise_userfeedback_smallmargin', label: data.canceltext, OnClicked: cancelfct}]);
        }

        doinit() {
            super.doinit();
            var self = this;
            setTimeout(function(){self.focus();});
        }

    }

    anise_internalUI.UserFeedbackWidget_choice = UserFeedbackWidget_choice;

    class UserFeedbackWidget_treechoice extends UserFeedbackWidget {

        _createnode(tn, node) {
            for(var i=0; i<tn.childs.length; i++) {
                var c = tn.childs[i];
                var nn = node.addRow([c.key]);
                nn.setMetadata("icon", clove.Icon.byUrl("static/icons/" + c.icon));
                nn.setMetadata("answer", c.val);
                this._createnode(c, nn);
            }
        }

        data_setter(data) {
            var self = this;
            var ds = new clove.NativeDatasource();
            this._createnode(data.choicetree, ds.root());
            this.setBody([
                {label: data.msg, styleClass: 'anise_userfeedback_scrollable anise_userfeedback_larger'},
                {view: 'TreeView', allowSelection: true, datasource: ds, name: 'tree', styleClass: 'anise_userfeedback_margin anise_userfeedback_scrollable', vstretch: 1, height: "1pt", hstretch: 1, width: "1pt"},
            ]);
            var buttons = [];
            var choosefct = function(){
                var s = self.getByName('tree').selection();
                if (s)
                    self.answerFeedback(s.backendObject.getMetadata().answer);
            };
            buttons.push({ view: 'anise_internalUI.UserFeedbackButton', shortcutText: this.addShortcut(choosefct, 'enter', 10), styleClass: 'anise_userfeedback_smallmargin', label: "Choose", OnClicked: choosefct});
            for(var i=0; i<data.actions.length; i++) {
                var fct = (function(_i){ return function(){self.answerFeedback(_i);}; })(-i);
                buttons.push({ view: 'anise_internalUI.UserFeedbackButton', shortcutText: this.addShortcut(fct), styleClass: 'anise_userfeedback_smallmargin', label: data.actions[i], OnClicked: fct});
            }
            var cancelfct = function(){self.answerFeedback(undefined);}
            buttons.push({ view: 'anise_internalUI.UserFeedbackButton', shortcutText: this.addShortcut(cancelfct, 'esc', 27), styleClass: 'anise_userfeedback_smallmargin', label: data.canceltext, OnClicked: cancelfct});
            this.setButtons(buttons);
        }

        doinit() {
            super.doinit();
            var self = this;
            setTimeout(function(){self.getByName('tree').focus();});
        }

    }

    anise_internalUI.UserFeedbackWidget_treechoice = UserFeedbackWidget_treechoice;

})();

yaji.ready(() => {

    yaji.addEventHandler('scrollDownPager', ()=>clove.getByName('pager').scrollDown());

    yaji.appconfig.addHandler('mainviewStyleClass', (v)=>{
        clove.getByName('mainview').setStyleClass(v);
    });

});
