# Copyright (C) 2014-2021, Josef Hahn
#
# This file is part of anise.
#
# anise is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# anise is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with anise.  If not, see <http://www.gnu.org/licenses/>.

import os
import shutil

import anise.framework.files
import anise.test.anisetest
import anise.utils


_projectdesc = """<?xml version="1.0" ?>
<aniseprojectdesc>
    <val k="name" v="myprojectname" />
    <val k="base.version" v="1.2" />
    <val k="base.summary" v="mysummary" />
    <val k="base.email" v="my@mail.address" />
    <val k="base.authors"><val v="John Doe" /></val>
    <val k="base.license" oref="licensing.PublicDomain" />
    <val k="base.maturity" oref="base.Maturity.ProductionStable" />
    <let k="base.set_basecolor">
        <val k="r" t="float" v="0.1" />
        <val k="g" t="float" v="0.1" />
        <val k="b" t="float" v="0.1" />
    </let>
    <val k="base.longdescription" v="mylongdescription" />
    <let k="distributables.pool.addgroup" name="source tarball">
        <val k="sources" t="files.TaskExecution">
            <val k="task" oref="myfeaturef4.mytarpackage" />
            <val k="source" t="files.TaskExecution">
                <val k="task" oref="packages.makerawpackage" />
            </val>
        </val>
    </let>
    <let k="releasing.tasks.add" destination="{0}">
        <val k="task" oref="filetransfer.localupload" />
        <val k="source" t="files.TaskExecution">
            <val k="task" oref="homepage.makehomepage" />
        </val>
    </let>
    <let k="packagestore.enable" />
    <val k="packagestore.historylength" t="int" v="3" />
</aniseprojectdesc>
"""

_myfeature = """
from anise.framework.projects import universe

def mytarpackage(*a,**b):
    if getattr(universe, "packagebroken", False):
        raise Exception("kaputt :-p")
    return universe.packages.tarpackage(*a,**b)
"""


class Test(anise.test.anisetest.AniseTestCase):

    def __init__(self, *a, **kwa):
        self.destination = anise.framework.files.TempDir().path
        super().__init__(*a, projectdesc=_projectdesc.format(self.destination), **kwa)

    def test_a(self):
        anise.utils.basic.mkdirs(self.rootdir + "/features")
        with open(self.rootdir + "/features/myfeaturef4.py", "w") as f:
            f.write(_myfeature)
        loadfeaturesfrom = ["builtin", self.rootdir + "/features"]
        self.runanise("releasing.release", values={'base.version': '1.2-1'}, loadfeaturesfrom=loadfeaturesfrom)
        self.runanise("releasing.release", values={'base.version': '1.5-2'}, loadfeaturesfrom=loadfeaturesfrom)
        self.runanise("releasing.release", values={'base.version': '1.3-3'}, loadfeaturesfrom=loadfeaturesfrom)
        self.runanise("releasing.release", values={'base.version': '1.4-4'}, loadfeaturesfrom=loadfeaturesfrom)
        self.runanise("releasing.release", values={'base.version': '1.3-5'}, loadfeaturesfrom=loadfeaturesfrom)
        self.runanise("releasing.release", values={'base.version': '1.4-6'}, loadfeaturesfrom=loadfeaturesfrom)
        self.runanise("releasing.release", values={'base.version': '1.5-7'}, loadfeaturesfrom=loadfeaturesfrom)
        self.runanise("releasing.release", values={'base.version': '1.2-8'}, loadfeaturesfrom=loadfeaturesfrom)
        p = self.destination
        shutil.rmtree(p + "/myprojectname")
        self.runanise("releasing.release", values={'base.version': '1.5-9'}, loadfeaturesfrom=loadfeaturesfrom)

        self.assertTrue(os.path.exists(self.packagestoredir +
                                       "/myprojectname/source tarball/1.2/myprojectname-1.2-8.tgz"))
        self.assertFalse(os.path.exists(p + "/myprojectname/myprojectname-1.2-1.tgz"))
        self.assertFalse(os.path.exists(p + "/myprojectname/myprojectname-1.2-8.tgz"))
        self.assertTrue(os.path.exists(p + "/myprojectname/myprojectname-1.3-5.tgz"))
        self.assertTrue(os.path.exists(p + "/myprojectname/myprojectname-1.4-6.tgz"))
        self.assertTrue(os.path.exists(p + "/myprojectname/myprojectname-1.5-9.tgz"))

        shutil.rmtree(p + "/myprojectname")

        with self.assertRaises(Exception):
            self.runanise("releasing.release", values={'base.version': '1.5-9', 'packagebroken': True},
                          loadfeaturesfrom=loadfeaturesfrom)

        self.runanise("releasing.release",
                      values={'base.version': '1.5-9', 'packagebroken': True, 'skippackagecreation': True},
                      loadfeaturesfrom=loadfeaturesfrom)

        self.assertTrue(os.path.exists(p + "/myprojectname/myprojectname-1.3-5.tgz"))
        self.assertTrue(os.path.exists(p + "/myprojectname/myprojectname-1.4-6.tgz"))
        self.assertTrue(os.path.exists(p + "/myprojectname/myprojectname-1.5-9.tgz"))
