# Copyright (C) 2014-2021, Josef Hahn
#
# This file is part of anise.
#
# anise is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# anise is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with anise.  If not, see <http://www.gnu.org/licenses/>.

import anise.framework.files
import anise.test.anisetest
import anise.utils


_projectdesc = """<?xml version="1.0" ?>
<aniseprojectdesc>
    <val k="name" v="myprojectname" />
    <val k="base.version" v="1.2" />
    <val k="base.summary" v="mysummary" />
    <val k="base.email" v="my@mail.address" />
    <val k="base.authors"><val v="John Doe" /></val>
    <val k="base.license" oref="licensing.PublicDomain" />
    <val k="base.maturity" oref="base.Maturity.ProductionStable" />
    <val k="base.longdescription" v="mylongdescription" />
</aniseprojectdesc>
"""

_myfeature = """
from anise.framework.projects import universe

def foo():
    return universe.build.make.build(source=universe.packages.RawPackage(), targetname="mytool")
"""


class Test(anise.test.anisetest.AniseTestCase):

    def __init__(self, *a, **kwa):
        self.destination = anise.framework.files.TempDir().path
        super().__init__(*a, projectdesc=_projectdesc.format(self.destination), **kwa)

    def test_a(self):
        anise.utils.basic.mkdirs(self.rootdir + "/features")
        anise.utils.basic.writetofile(self.rootdir + "/features/myfeature.py", _myfeature)
        loadfeaturesfrom = ["builtin", self.rootdir + "/features"]
        anise.utils.basic.writetofile(self.mydir + "/mytool.c", """
#include <stdio.h>
main() {
    printf("boom");
}
        """)
        anise.utils.basic.writetofile(self.mydir + "/Makefile", """
mytool:
\tgcc mytool.c -o mytool
        """)
        with anise.utils.basic.ChDir(anise.framework.files.TempDir().path):
            mytool = self.runanise("myfeature.foo", loadfeaturesfrom=loadfeaturesfrom).path
            r, o = anise.utils.basic.call(mytool)
        self.assertEqual(o, "boom")
