# Copyright (C) 2014-2021, Josef Hahn
#
# This file is part of anise.
#
# anise is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# anise is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with anise.  If not, see <http://www.gnu.org/licenses/>.

"""
Helpers for creation and handling of some data structures.
"""

from typing import *


def constructobject(**d: Optional[Any]) -> object:
    """
    Creates an object with properties from the given keyword arguments.
    """
    class Aniseobject:
        pass
    r = Aniseobject()
    for k in d:
        setattr(r, k, d[k])
    return r


def createobjectstructure(root: Any, newnames: str, objclass: Optional[Type] = None) -> Tuple[object, object]:
    """
    Creates a chain of new objects, so you can access property chains like :samp:`foo.bar.baz.bam` on the root object.

    :param root: The object to populate with the new structure.
    :param newnames: The complete property chain.
    :param objclass: Which object class to use for the cascaded new members.
    :return: Tuple of the deepest object in the cascade (corresponding to the last part of the chain) and its parent.
    """
    if objclass is None:
        class _AniseObject:
            pass
        objclass = _AniseObject
    parent = root
    parentparent = None
    for newname in newnames.split("."):
        parentparent = parent
        if hasattr(parent, newname):
            parent = getattr(parent, newname)
        else:
            _parent = objclass()
            setattr(parent, newname, _parent)
            parent = _parent
    return parent, parentparent
